@@ -1,7 +1,7 @@
 /*
  * ndpi_api.h
  *
- * Copyright (C) 2011-24 - ntop.org
+ * Copyright (C) 2011-25 - ntop.org
  *
  * This file is part of nDPI, an open source deep packet inspection
  * library based on the OpenDPI and PACE technology by ipoque GmbH
@@ -109,11 +109,14 @@ extern "C" {
   void * ndpi_calloc(unsigned long count, size_t size);
   void * ndpi_realloc(void *ptr, size_t old_size, size_t new_size);
   char * ndpi_strdup(const char *s);
+  char * ndpi_strndup(const char *s, size_t size);
   void   ndpi_free(void *ptr);
   void * ndpi_flow_malloc(size_t size);
   void   ndpi_flow_free(void *ptr);
   u_int32_t ndpi_get_tot_allocated_memory(void);
 
+  char *ndpi_strip_leading_trailing_spaces(char *ptr, int *ptr_len) ;
+
   /**
    * Finds the first occurrence of the substring 'needle' in the string 'haystack'.
    *
@@ -150,6 +153,8 @@ extern "C" {
    */
   u_int16_t ndpi_network_ptree_match(struct ndpi_detection_module_struct *ndpi_struct,
 				     struct in_addr *pin);
+  u_int16_t ndpi_network_ptree6_match(struct ndpi_detection_module_struct *ndpi_str,
+				      struct in6_addr *pin);
 
   /**
    * Returns the nDPI protocol id for IP+port-based protocol detection
@@ -334,7 +339,7 @@ extern "C" {
 				 const unsigned char *packet,
 				 const unsigned short packetlen,
 				 const u_int64_t packet_time_ms,
-				 const struct ndpi_flow_input_info *input_info);
+				 struct ndpi_flow_input_info *input_info);
 
   /**
    * Processes one packet and returns the ID of the detected protocol.
@@ -354,7 +359,7 @@ extern "C" {
 					      const unsigned char *packet,
 					      const unsigned short packetlen,
 					      const u_int64_t packet_time_ms,
-					      const struct ndpi_flow_input_info *input_info);
+					      struct ndpi_flow_input_info *input_info);
   /**
    * Get the main protocol of the passed flows for the detected module
    *
@@ -483,6 +488,7 @@ extern "C" {
    * @par    string_to_match_len = the length of the string
    * @par    ret_match           = completed returned match information
    * @par    master_protocol_id  = value of the ID associated to the master protocol detected
+   * @par    update_flow_classification = update or not protocol (sub)classification
    * @return the ID of the matched subprotocol
    *
    */
@@ -491,7 +497,8 @@ extern "C" {
 					char *string_to_match,
 					u_int string_to_match_len,
 					ndpi_protocol_match_result *ret_match,
-					u_int16_t master_protocol_id);
+					u_int16_t master_protocol_id,
+					int update_flow_classification);
 
   /**
    * Check if the string content passed match with a protocol
@@ -582,6 +589,25 @@ extern "C" {
   void ndpi_set_proto_category(struct ndpi_detection_module_struct *ndpi_mod,
 			       u_int16_t protoId, ndpi_protocol_category_t protoCategory);
 
+  /**
+   * Find the QoE category for the specified protocol
+   *
+   * @par     ndpi_mod      = the detection module
+   * @par     protoId       = the protocol identifier we're searhing
+   *
+   */
+  ndpi_protocol_qoe_category_t ndpi_find_protocol_qoe(struct ndpi_detection_module_struct *ndpi_str,
+						      u_int16_t protoId);
+
+  /**
+   * Return the name of a RTP payload type
+   *
+   * @par     payload_type     = the RTP payload type
+   * @par     evs_payload_type = EVS payload type (only in case payload_type is EVS)
+   * @return  The symbolic payload type or "Unknown" if not found
+   */
+  const char* ndpi_rtp_payload_type2str(u_int8_t payload_type, u_int32_t evs_payload_type);
+
   /**
    * Check if subprotocols of the specified master protocol are just
    * informative (and not real)
@@ -690,14 +716,34 @@ extern "C" {
   char* ndpi_get_proto_breed_name(ndpi_protocol_breed_t breed_id);
 
   /**
-   * Return the ID of the protocol
+   * Return the name of the protocol given its ID.
    *
    * @par     ndpi_mod   = the detection module
-   * @par     proto      = the protocol name
+   * @par     name       = the protocol name. You can specify TLS or YouYube but not TLS.YouTube (se ndpi_get_protocol_by_name in this case)
    * @return  the ID of the protocol
    *
    */
-  int ndpi_get_protocol_id(struct ndpi_detection_module_struct *ndpi_mod, char *proto);
+  extern u_int16_t ndpi_get_proto_by_name(struct ndpi_detection_module_struct *ndpi_mod, const char *name);
+
+  /**
+   * Return the name of the protocol given its ID
+   *
+   * @par     ndpi_mod   = the detection module
+   * @par     id         = the protocol id
+   * @return  the name of the protocol
+   *
+   */
+  extern char* ndpi_get_proto_by_id(struct ndpi_detection_module_struct *ndpi_mod, u_int id);
+
+  /**
+   * Return the name of the protocol given its ID. You can specify TLS.YouTube or just TLS
+   *
+   * @par     ndpi_mod   = the detection module
+   * @par     id         = the protocol id
+   * @return  the name of the protocol
+   *
+   */
+  extern ndpi_master_app_protocol ndpi_get_protocol_by_name(struct ndpi_detection_module_struct *ndpi_str, const char *name);
 
   /**
    * Return the ID of the category
@@ -827,14 +873,14 @@ extern "C" {
   int ndpi_load_risk_domain_file(struct ndpi_detection_module_struct *ndpi_str, const char* path);
 
   /**
-   * Read a file and load the list of malicious JA3 signatures
+   * Read a file and load the list of malicious JA4 signatures
    *
    * @par     ndpi_mod = the detection module
    * @par     path     = the path of the file
    * @return  0 if the file is loaded correctly;
    *          -1 else
    */
-  int ndpi_load_malicious_ja3_file(struct ndpi_detection_module_struct *ndpi_str, const char *path);
+  int ndpi_load_malicious_ja4_file(struct ndpi_detection_module_struct *ndpi_str, const char *path);
 
   /**
    * Read a file and load the list of malicious SSL certificate SHA1 fingerprints.
@@ -845,6 +891,27 @@ extern "C" {
    */
   int ndpi_load_malicious_sha1_file(struct ndpi_detection_module_struct *ndpi_str, const char *path);
 
+  /*
+    Add a new TCP fingerprint
+
+    Return code:
+    0   OK
+    -1  Duplicated fingerprint
+    -2  Unable to add a new entry
+  */
+  int ndpi_add_tcp_fingerprint(struct ndpi_detection_module_struct *ndpi_str,
+			       char *fingerprint, ndpi_os os);
+
+  /**
+   * Read a file and load the list of TCP fingerprints
+   * @par     ndpi_mod = the detection module
+   * @par     path     = the path of the file
+   * @return  0 if the file is loaded correctly;
+   *          -1 else
+   */
+  int load_tcp_fingerprint_file_fd(struct ndpi_detection_module_struct *ndpi_str, FILE *fd);
+  int ndpi_load_tcp_fingerprint_file(struct ndpi_detection_module_struct *ndpi_str, const char *path);
+
   /**
    * Get the total number of the supported protocols
    *
@@ -1036,6 +1103,9 @@ extern "C" {
 
   u_int16_t ndpi_get_lower_proto(ndpi_protocol proto);
   u_int16_t ndpi_get_upper_proto(ndpi_protocol proto);
+  bool ndpi_is_proto(ndpi_master_app_protocol proto, u_int16_t p);
+  bool ndpi_is_proto_unknown(ndpi_master_app_protocol proto);
+  bool ndpi_is_proto_equals(ndpi_master_app_protocol to_check, ndpi_master_app_protocol to_match, bool exact_match_only);
 
   ndpi_proto_defaults_t* ndpi_get_proto_defaults(struct ndpi_detection_module_struct *ndpi_mod);
   u_int ndpi_get_ndpi_num_supported_protocols(struct ndpi_detection_module_struct *ndpi_mod);
@@ -1107,6 +1177,7 @@ extern "C" {
                                  ndpi_protocol const * const l7_protocol);
   char* ndpi_ssl_version2str(char *buf, int buf_len,
                              u_int16_t version, u_int8_t *unknown_tls_version);
+  char *ndpi_multimedia_flowtype2str(char *buf, int buf_len, u_int8_t m_types);
   char *ndpi_quic_version2str(char *buf, int buf_len, u_int32_t version);
   int ndpi_netbios_name_interpret(u_char *in, u_int in_len, u_char *out, u_int out_len);
   void ndpi_patchIPv6Address(char *str);
@@ -1177,6 +1248,7 @@ extern "C" {
   void ndpi_ptree_destroy(ndpi_ptree_t *tree);
 
   /* General purpose utilities */
+  u_int8_t ndpi_is_public_ipv4(u_int32_t a /* host byte order */);
   u_int64_t ndpi_htonll(u_int64_t v);
   u_int64_t ndpi_ntohll(u_int64_t v);
   u_int8_t ndpi_is_valid_protoId(u_int16_t protoId);
@@ -1185,7 +1257,8 @@ extern "C" {
   /* DGA */
   int ndpi_check_dga_name(struct ndpi_detection_module_struct *ndpi_str,
 			  struct ndpi_flow_struct *flow,
-			  char *name, u_int8_t is_hostname, u_int8_t check_subproto);
+			  char *name, u_int8_t is_hostname, u_int8_t check_subproto,
+			  u_int8_t flow_fully_classified);
 
   /* Serializer (supports JSON, TLV, CSV) */
 
@@ -1700,6 +1773,7 @@ extern "C" {
   float ndpi_data_variance(struct ndpi_analyze_struct *s);
   float ndpi_data_stddev(struct ndpi_analyze_struct *s);
   float ndpi_data_mean(struct ndpi_analyze_struct *s);
+  float ndpi_data_jitter(struct ndpi_analyze_struct *s);
   u_int64_t ndpi_data_last(struct ndpi_analyze_struct *s);
   u_int64_t ndpi_data_min(struct ndpi_analyze_struct *s);
   u_int64_t ndpi_data_max(struct ndpi_analyze_struct *s);
@@ -1730,12 +1804,12 @@ extern "C" {
 
   void ndpi_md5(const u_char *data, size_t data_len, u_char hash[16]);
   void ndpi_sha256(const u_char *data, size_t data_len, u_int8_t sha_hash[32]);
-  
+
   u_int16_t ndpi_crc16_ccit(const void* data, size_t n_bytes);
   u_int16_t ndpi_crc16_ccit_false(const void *data, size_t n_bytes);
   u_int16_t ndpi_crc16_xmodem(const void *data, size_t n_bytes);
   u_int16_t ndpi_crc16_x25(const void* data, size_t n_bytes);
-  u_int32_t ndpi_crc32(const void* data, size_t n_bytes);
+  u_int32_t ndpi_crc32(const void *data, size_t length, u_int32_t crc);
   u_int32_t ndpi_nearest_power_of_two(u_int32_t x);
 
   /* ******************************* */
@@ -1749,6 +1823,11 @@ extern "C" {
 
   /* ******************************* */
 
+  u_int ndpi_hex2bin(u_char *out, u_int out_len, u_char* in, u_int in_len);
+  u_int ndpi_bin2hex(u_char *out, u_int out_len, u_char* in, u_int in_len);
+
+  /* ******************************* */
+
   int ndpi_des_init(struct ndpi_des_struct *des, double alpha, double beta, float significance);
   int ndpi_des_add_value(struct ndpi_des_struct *des, const double _value, double *forecast, double *confidence_band);
   void ndpi_des_fitting(double *values, u_int32_t num_values, float *ret_alpha, float *ret_beta);
@@ -1778,11 +1857,13 @@ extern "C" {
                             ndpi_confidence_t confidence,
                             ndpi_protocol l7_protocol);
   const char* ndpi_risk2str(ndpi_risk_enum risk);
+  const char* ndpi_risk2code(ndpi_risk_enum risk);
+  ndpi_risk_enum ndpi_code2risk(const char* risk);
   const char* ndpi_severity2str(ndpi_risk_severity s);
   ndpi_risk_info* ndpi_risk2severity(ndpi_risk_enum risk);
   u_int16_t ndpi_risk2score(ndpi_risk risk,
 			    u_int16_t *client_score, u_int16_t *server_score);
-
+  char* print_ndpi_address_port(ndpi_address_port *ap, char *buf, u_int buf_len);
   u_int8_t ndpi_check_issuerdn_risk_exception(struct ndpi_detection_module_struct *ndpi_str,
 					      char *issuerDN);
   u_int8_t ndpi_check_flow_risk_exceptions(struct ndpi_detection_module_struct *ndpi_str,
@@ -1823,6 +1904,11 @@ extern "C" {
 
   /* ******************************* */
 
+  /* Mahalanobis distance (https://en.wikipedia.org/wiki/Mahalanobis_distance) between a point x and a distribution with mean u and inverted covariant matrix i_s */
+  float ndpi_mahalanobis_distance(const u_int32_t *x, u_int32_t size, const float *u, const float *i_s);
+
+  /* ******************************* */
+
   int  ndpi_init_bin(struct ndpi_bin *b, enum ndpi_bin_family f, u_int16_t num_bins);
   void ndpi_free_bin(struct ndpi_bin *b);
   struct ndpi_bin* ndpi_clone_bin(struct ndpi_bin *b);
@@ -1840,6 +1926,59 @@ extern "C" {
 
   /* ******************************* */
 
+  /* create a kd-tree for num_dimensions vector items */
+  ndpi_kd_tree* ndpi_kd_create(u_int num_dimensions);
+
+  /* free the ndpi_kd_tree */
+  void ndpi_kd_free(ndpi_kd_tree *tree);
+
+  /* remove all the elements from the tree */
+  void ndpi_kd_clear(ndpi_kd_tree *tree);
+
+  /* insert a node, specifying its position, and optional data.
+     Return true = OK, false otherwise
+  */
+  bool ndpi_kd_insert(ndpi_kd_tree *tree, const double *data_vector, void *user_data);
+
+  /* Find the nearest node from a given point.
+   * This function returns a pointer to a result set with at most one element.
+   */
+  ndpi_kd_tree_result *ndpi_kd_nearest(ndpi_kd_tree *tree, const double *data_vector);
+
+  /* returns the size of the result set (in elements) */
+  u_int32_t ndpi_kd_num_results(ndpi_kd_tree_result *res);
+
+  /* returns the current element and updates user_data with the data put during insert */
+  double* ndpi_kd_result_get_item(ndpi_kd_tree_result *res, double **user_data);
+
+  /* frees a result set returned by kd_nearest_range() */
+  void ndpi_kd_result_free(ndpi_kd_tree_result *res);
+
+  /* Returns the distance (square root of the individual elements difference) */
+  double ndpi_kd_distance(double *a1, double *b2, u_int num_dimensions);
+
+  /* ******************************* */
+
+  /*
+    Ball Tree: similar to KD-tree but more efficient with high cardinalities
+
+    - https://en.wikipedia.org/wiki/Ball_tree
+    - https://www.geeksforgeeks.org/ball-tree-and-kd-tree-algorithms/
+    - https://varshasaini.in/kd-tree-and-ball-tree-knn-algorithm/
+    - https://varshasaini.in/k-nearest-neighbor-knn-algorithm-in-machine-learning/
+
+    NOTE:
+    with ball tree, data is a vector of vector pointers (no array)
+  */
+  ndpi_btree* ndpi_btree_init(double **data, u_int32_t n_rows, u_int32_t n_columns);
+  ndpi_knn ndpi_btree_query(ndpi_btree *b, double **query_data,
+			    u_int32_t query_data_num_rows, u_int32_t query_data_num_columns,
+			    u_int32_t max_num_results);
+  void ndpi_free_knn(ndpi_knn knn);
+  void ndpi_free_btree(ndpi_btree *tree);
+
+  /* ******************************* */
+
   /*
    * Finds outliers using Z-score
    * Z-Score = (Value - Mean) / StdDev
@@ -1887,9 +2026,9 @@ extern "C" {
    *
    */
   double ndpi_pearson_correlation(u_int32_t *values_a, u_int32_t *values_b, u_int16_t num_values);
-  
+
   /* ******************************* */
-  
+
   /*
    * Checks if a specified value is an outlier with respect to past values
    * using the Z-score.
@@ -1901,12 +2040,12 @@ extern "C" {
    *                        t = 1 - The value to check should not exceed the past values
    *                        t > 1 - The value to check has to be within (t * stddev) boundaries
    * @par lower           - [out] Lower threshold
-   * @par upper           - [out] Upper threshold   
+   * @par upper           - [out] Upper threshold
    *
    * @return true if the specified value is an outlier, false otherwise
    *
    */
-  
+
   bool ndpi_is_outlier(u_int32_t *past_values, u_int32_t num_past_values,
 		       u_int32_t value_to_check, float threshold,
 		       float *lower, float *upper);
@@ -1969,9 +2108,15 @@ extern "C" {
   void ndpi_free_geoip(struct ndpi_detection_module_struct *ndpi_str);
   int ndpi_get_geoip_asn(struct ndpi_detection_module_struct *ndpi_str,
 			 char *ip, u_int32_t *asn);
+  int ndpi_get_geoip_aso(struct ndpi_detection_module_struct *ndpi_str,
+			 char *ip, char *aso, u_int8_t aso_len);
   int ndpi_get_geoip_country_continent(struct ndpi_detection_module_struct *ndpi_str, char *ip,
 				       char *country_code, u_int8_t country_code_len,
 				       char *continent, u_int8_t continent_len);
+  int ndpi_get_geoip_country_continent_city(struct ndpi_detection_module_struct *ndpi_str, char *ip,
+					    char *country_code, u_int8_t country_code_len,
+					    char *continent, u_int8_t continent_len,
+					    char *city, u_int8_t city_len);
 
   /* ******************************* */
 
@@ -2088,7 +2233,7 @@ extern "C" {
 				     ndpi_domain_classify *s,
 				     u_int16_t *class_id /* out */,
 				     char *hostname);
-  
+
   /* ******************************* */
 
   /*
@@ -2210,8 +2355,9 @@ extern "C" {
   int64_t ndpi_strtonum(const char *numstr, int64_t minval, int64_t maxval, const char **errstrp, int base);
   int ndpi_vsnprintf(char * str, size_t size, char const * format, va_list va_args);
   int ndpi_snprintf(char * str, size_t size, char const * format, ...);
-  struct tm *ndpi_gmtime_r(const time_t *timep,
-                           struct tm *result);
+  struct tm *ndpi_gmtime_r(const time_t *timep, struct tm *result);
+  char* ndpi_strrstr(const char *haystack, const char *needle);
+  int ndpi_str_endswith(const char *s, const char *suffix);
 
   /* ******************************* */
 
@@ -2226,7 +2372,49 @@ extern "C" {
   */
   u_int ndpi_encode_domain(struct ndpi_detection_module_struct *ndpi_str,
 			   char *domain, char *out, u_int out_len);
-    
+
+  /* ******************************* */
+
+  char* ndpi_quick_encrypt(const char *cleartext_msg,
+			   u_int16_t cleartext_msg_len,
+			   u_int16_t *encrypted_msg_len,
+			   u_char encrypt_key[64]);
+
+  char* ndpi_quick_decrypt(const char *encrypted_msg,
+			   u_int16_t encrypted_msg_len,
+			   u_int16_t *decrypted_msg_len,
+			   u_char decrypt_key[64]);
+
+  /* ******************************* */
+
+  const char* ndpi_print_os_hint(ndpi_os os_hint);
+
+  /* ******************************* */
+
+  bool ndpi_serialize_flow_fingerprint(struct ndpi_detection_module_struct *ndpi_str,
+				       struct ndpi_flow_struct *flow, ndpi_serializer *serializer);
+
+  /* ******************************* */
+
+  /* Address cache API */
+  struct ndpi_address_cache* ndpi_init_address_cache(u_int32_t max_num_entries);
+  void ndpi_term_address_cache(struct ndpi_address_cache *cache);
+  u_int32_t ndpi_address_cache_flush_expired(struct ndpi_address_cache *cache, u_int32_t epoch_now);
+  struct ndpi_address_cache_item* ndpi_address_cache_find(struct ndpi_address_cache *cache, ndpi_ip_addr_t ip_addr, u_int32_t epoch_now);
+  bool ndpi_address_cache_insert(struct ndpi_address_cache *cache, ndpi_ip_addr_t ip_addr, char *hostname,
+				 u_int32_t epoch_now, u_int32_t ttl);
+  bool ndpi_address_cache_dump(struct ndpi_address_cache *cache, char *path, u_int32_t epoch_now);
+  u_int32_t ndpi_address_cache_restore(struct ndpi_address_cache *cache, char *path, u_int32_t epoch_now);
+
+
+  bool ndpi_cache_address(struct ndpi_detection_module_struct *ndpi_struct,
+			ndpi_ip_addr_t ip_addr, char *hostname,
+			  u_int32_t epoch_now, u_int32_t ttl);
+  struct ndpi_address_cache_item* ndpi_cache_address_find(struct ndpi_detection_module_struct *ndpi_struct, ndpi_ip_addr_t ip_addr);
+  bool ndpi_cache_address_dump(struct ndpi_detection_module_struct *ndpi_struct, char *path, u_int32_t epoch_now);
+  u_int32_t ndpi_cache_address_restore(struct ndpi_detection_module_struct *ndpi_struct, char *path, u_int32_t epoch_now);
+  u_int32_t ndpi_cache_address_flush_expired(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t epoch_now);
+
   /* ******************************* */
 
   const char *ndpi_lru_cache_idx_to_name(lru_cache_type idx);
@@ -2267,7 +2455,20 @@ extern "C" {
    * @return Length of src string
    */
   size_t ndpi_strlcpy(char* dst, const char* src, size_t dst_len, size_t src_len);
-
+  
+  /**
+   * @brief Converts a string from ISO 8859 to UTF-8
+   *
+   * @param in String to convert
+   * @param in_len Source string lenght
+   * @param out Destination string buffer (UTF-8)
+   * @param out_len Length of destination string buffer. It must be at least (2*in_len)+1
+   *
+   * @return The destination string buffer
+   */
+  u_char* ndpi_str_to_utf8(u_char *in, u_int in_len, u_char *out, u_int out_len);
+  
+    
 #ifdef __cplusplus
 }
 #endif
