@@ -1,7 +1,7 @@
 /*
  * ndpi_main.h
  *
- * Copyright (C) 2011-22 - ntop.org
+ * Copyright (C) 2011-25 - ntop.org
  *
  * This file is part of nDPI, an open source deep packet inspection
  * library based on the OpenDPI and PACE technology by ipoque GmbH
@@ -69,11 +69,6 @@ extern "C" {
   extern char *ndpi_get_ip_string(const ndpi_ip_addr_t * ip, char *buf, u_int buf_len);
   extern u_int8_t ndpi_is_ipv6(const ndpi_ip_addr_t *ip);
 
-  extern char* ndpi_get_proto_by_id(struct ndpi_detection_module_struct *ndpi_mod, u_int id);
-  u_int16_t ndpi_get_proto_by_name(struct ndpi_detection_module_struct *ndpi_mod, const char *name);
-
-  extern u_int8_t ndpi_is_proto(ndpi_protocol proto, u_int16_t p);
-
   extern void ndpi_search_tcp_or_udp(struct ndpi_detection_module_struct *ndpi_struct, struct ndpi_flow_struct *flow);
 
   /** Checks when the @p payload starts with the string literal @p str.
@@ -96,11 +91,12 @@ extern "C" {
 			       ndpi_protocol_breed_t breed,
 			       u_int16_t protoId, char *protoName,
 			       ndpi_protocol_category_t protoCategory,
+			       ndpi_protocol_qoe_category_t qoeCategory,
 			       ndpi_port_range *tcpDefPorts,
 			       ndpi_port_range *udpDefPorts);
-  void ndpi_set_risk(struct ndpi_flow_struct *flow, ndpi_risk_enum r,
-		     char *risk_message);
-  void ndpi_unset_risk(struct ndpi_flow_struct *flow, ndpi_risk_enum r);
+  void ndpi_set_risk(struct ndpi_detection_module_struct *ndpi_str, struct ndpi_flow_struct *flow,
+                     ndpi_risk_enum r, char *risk_message);
+  void ndpi_unset_risk(struct ndpi_detection_module_struct *ndpi_str, struct ndpi_flow_struct *flow, ndpi_risk_enum r);
   int ndpi_isset_risk(struct ndpi_flow_struct *flow, ndpi_risk_enum r);
   int ndpi_is_printable_buffer(u_int8_t const * const buf, size_t len);
   int ndpi_normalize_printable_string(char * const str, size_t len);
@@ -113,7 +109,7 @@ extern "C" {
 #define NDPI_ENTROPY_ENCRYPTED_OR_RANDOM(entropy) (entropy >= 7.312f)
   float ndpi_entropy(u_int8_t const * const buf, size_t len);
   char *ndpi_entropy2str(float entropy, char *buf, size_t len);
-  void ndpi_entropy2risk(struct ndpi_flow_struct *flow);
+  void ndpi_entropy2risk(struct ndpi_detection_module_struct *ndpi_struct, struct ndpi_flow_struct *flow);
 
 #ifdef __cplusplus
 }
