@@ -1,7 +1,7 @@
 /*
  * ndpi_protocol_ids.h
  *
- * Copyright (C) 2011-22 - ntop.org
+ * Copyright (C) 2011-25 - ntop.org
  *
  * This file is part of nDPI, an open source deep packet inspection
  * library based on the OpenDPI and PACE technology by ipoque GmbH
@@ -66,7 +66,7 @@ typedef enum {
   NDPI_PROTOCOL_GNUTELLA              = 35,
   NDPI_PROTOCOL_EDONKEY               = 36,
   NDPI_PROTOCOL_BITTORRENT            = 37,
-  NDPI_PROTOCOL_SKYPE_TEAMS_CALL      = 38, /* Skype call and videocalls */
+  NDPI_PROTOCOL_MSTEAMS_CALL          = 38, /* MS Teams (and Skype) videocalls */
   NDPI_PROTOCOL_SIGNAL                = 39,
   NDPI_PROTOCOL_MEMCACHED             = 40, /* Memcached */
   NDPI_PROTOCOL_SMBV23                = 41, /* SMB version 2/3 */
@@ -141,7 +141,7 @@ typedef enum {
   NDPI_PROTOCOL_AMAZON_ALEXA          = 110,
   NDPI_PROTOCOL_KERBEROS              = 111,
   NDPI_PROTOCOL_LDAP                  = 112,
-  NDPI_PROTOCOL_MAPLESTORY            = 113,
+  NDPI_PROTOCOL_NEXON                 = 113,
   NDPI_PROTOCOL_MSSQL_TDS             = 114,
   NDPI_PROTOCOL_PPTP                  = 115,
   NDPI_PROTOCOL_WARCRAFT3             = 116,
@@ -153,7 +153,7 @@ typedef enum {
   NDPI_PROTOCOL_GMAIL                 = 122,
   NDPI_PROTOCOL_GOOGLE_MAPS           = 123,
   NDPI_PROTOCOL_YOUTUBE               = 124,
-  NDPI_PROTOCOL_SKYPE_TEAMS           = 125,
+  NDPI_PROTOCOL_MOZILLA               = 125,
   NDPI_PROTOCOL_GOOGLE                = 126,
   NDPI_PROTOCOL_MS_RPCH               = 127,
   NDPI_PROTOCOL_NETFLOW               = 128,
@@ -241,7 +241,7 @@ typedef enum {
   NDPI_PROTOCOL_DEEZER                = 210,
   NDPI_PROTOCOL_INSTAGRAM             = 211,
   NDPI_PROTOCOL_MICROSOFT             = 212,
-  NDPI_PROTOCOL_STARCRAFT             = 213,
+  NDPI_PROTOCOL_BLIZZARD              = 213,
   NDPI_PROTOCOL_TEREDO                = 214,
   NDPI_PROTOCOL_HOTSPOT_SHIELD        = 215,
   NDPI_PROTOCOL_IMO                   = 216,
@@ -408,7 +408,7 @@ typedef enum {
   NDPI_PROTOCOL_APACHE_KAFKA          = 377,
   NDPI_PROTOCOL_NOMACHINE             = 378,
   NDPI_PROTOCOL_IEC62056              = 379,
-  NDPI_PROTOCOL_HL7                   = 380,
+  NDPI_PROTOCOL_HL7                   = 380, /* Hospital Protocol */
   NDPI_PROTOCOL_CEPH                  = 381,
   NDPI_PROTOCOL_GOOGLE_CHAT           = 382,
   NDPI_PROTOCOL_ROUGHTIME             = 383,
@@ -450,6 +450,32 @@ typedef enum {
   NDPI_PROTOCOL_CLICKHOUSE            = 419,
   NDPI_PROTOCOL_NANO                  = 420,
   NDPI_PROTOCOL_OPENWIRE              = 421,
+  NDPI_PROTOCOL_CNP_IP                = 422,
+  NDPI_PROTOCOL_ATG                   = 423,
+  NDPI_PROTOCOL_TRDP                  = 424,
+  NDPI_PROTOCOL_LUSTRE                = 425,
+  NDPI_PROTOCOL_NORDVPN               = 426,
+  NDPI_PROTOCOL_SURFSHARK             = 427,
+  NDPI_PROTOCOL_CACTUSVPN             = 428,
+  NDPI_PROTOCOL_WINDSCRIBE            = 429,
+  NDPI_PROTOCOL_SONOS                 = 430,
+  NDPI_PROTOCOL_DINGTALK              = 431,
+  NDPI_PROTOCOL_PALTALK               = 432,
+  NDPI_PROTOCOL_NAVER                 = 433,
+  NDPI_PROTOCOL_SHEIN                 = 434,
+  NDPI_PROTOCOL_TEMU                  = 435,
+  NDPI_PROTOCOL_TAOBAO                = 436,
+  NDPI_PROTOCOL_MIKROTIK              = 437,
+  NDPI_PROTOCOL_DICOM                 = 438, /* Hospital Protocol */
+  NDPI_PROTOCOL_PARAMOUNTPLUS         = 439,
+  NDPI_PROTOCOL_YANDEX_ALICE          = 440,
+  NDPI_PROTOCOL_VIVOX                 = 441,
+  NDPI_PROTOCOL_DIGITALOCEAN          = 442,
+  NDPI_PROTOCOL_RUTUBE                = 443,
+  NDPI_PROTOCOL_LAGOFAST              = 444,
+  NDPI_PROTOCOL_GEARUP_BOOSTER        = 445,
+  NDPI_PROTOCOL_LLM                   = 446, /* Large Language Models */
+  NDPI_PROTOCOL_UBIQUITY              = 447,
 
 #ifdef CUSTOM_NDPI_PROTOCOLS
 #include "../../../nDPI-custom/custom_ndpi_protocol_ids.h"
