@@ -1,7 +1,7 @@
 /*
  * ndpi_typedefs.h
  *
- * Copyright (C) 2011-23 - ntop.org
+ * Copyright (C) 2011-25 - ntop.org
  *
  * This file is part of nDPI, an open source deep packet inspection
  * library based on the OpenDPI and PACE technology by ipoque GmbH
@@ -41,8 +41,6 @@
 /* Used by both nDPI core and patricia code under third-party */
 #include "ndpi_patricia_typedefs.h"
 
-// #define USE_LEGACY_AHO_CORASICK
-
 #ifndef NDPI_CFFI_PREPROCESSING
 #ifndef u_char
 typedef unsigned char u_char;
@@ -72,10 +70,10 @@ typedef enum {
 } ndpi_log_level_t;
 
 typedef enum {
-  ndpi_multimedia_unknown_flow = 0,
-  ndpi_multimedia_audio_flow,
-  ndpi_multimedia_video_flow,
-  ndpi_multimedia_screen_sharing_flow,
+  ndpi_multimedia_unknown_flow          = 0x00,
+  ndpi_multimedia_audio_flow            = 0x01,
+  ndpi_multimedia_video_flow            = 0x02,
+  ndpi_multimedia_screen_sharing_flow   = 0x04,
 } ndpi_multimedia_flow_type;
 
 typedef enum {
@@ -99,7 +97,7 @@ typedef enum {
   NOTE
   When the typedef below is modified don't forget to update
   - nDPI/wireshark/ndpi.lua
-  - ndpi_risk2str (in ndpi_utils.c)
+  - ndpi_risk2str, ndpi_risk2code, ndpi_code2risk, ndpi_risk_shortnames (in ndpi_utils.c)
   - doc/flow_risks.rst
   - ndpi_known_risks (ndpi_main.c)
 
@@ -132,14 +130,14 @@ typedef enum {
   NDPI_SSH_OBSOLETE_CLIENT_VERSION_OR_CIPHER,
   NDPI_SSH_OBSOLETE_SERVER_VERSION_OR_CIPHER,
   NDPI_SMB_INSECURE_VERSION, /* 20 */
-  NDPI_TLS_SUSPICIOUS_ESNI_USAGE,
+  NDPI_FREE_21,                                          /* FREE */
   NDPI_UNSAFE_PROTOCOL,
   NDPI_DNS_SUSPICIOUS_TRAFFIC,
   NDPI_TLS_MISSING_SNI,
   NDPI_HTTP_SUSPICIOUS_CONTENT,
   NDPI_RISKY_ASN,
   NDPI_RISKY_DOMAIN,
-  NDPI_MALICIOUS_JA3,
+  NDPI_MALICIOUS_FINGERPRINT,
   NDPI_MALICIOUS_SHA1_CERTIFICATE,
   NDPI_DESKTOP_OR_FILE_SHARING_SESSION, /* 30 */
   NDPI_TLS_UNCOMMON_ALPN,
@@ -163,19 +161,24 @@ typedef enum {
   NDPI_HTTP_OBSOLETE_SERVER,
   NDPI_PERIODIC_FLOW,          /* Set in case a flow repeats at a specific pace [used by apps on top of nDPI] */
   NDPI_MINOR_ISSUES,           /* Generic packet issues (e.g. DNS with 0 TTL) */
-  NDPI_TCP_ISSUES,             /* 50 */ /* TCP issues such as connection failed or scan */
-  NDPI_FULLY_ENCRYPTED,        /* This (unknown) session is fully encrypted */
+  NDPI_TCP_ISSUES,    /* 50 */ /* TCP issues such as connection failed or scan */
+  NDPI_FREE_51,                                         /* FREE */
   NDPI_TLS_ALPN_SNI_MISMATCH,  /* Invalid ALPN/SNI combination */
   NDPI_MALWARE_HOST_CONTACTED, /* Flow client contacted a malware host */
   NDPI_BINARY_DATA_TRANSFER,   /* Attempt to transfer something in binary format */
   NDPI_PROBING_ATTEMPT,        /* Probing attempt (e.g. TCP connection with no data exchanged or unidirection traffic for bidirectional flows such as SSH) */
-  
+  NDPI_OBFUSCATED_TRAFFIC,
+  /* Before allocating a new risk here, check if there are FREE entries above */
+
   /* Leave this as last member */
   NDPI_MAX_RISK /* must be <= 63 due to (**) */
 } ndpi_risk_enum;
 
 typedef u_int64_t ndpi_risk; /* (**) */
 
+/*Used mainly by configuration */
+extern const char *ndpi_risk_shortnames[NDPI_MAX_RISK];
+
 typedef enum {
   NDPI_PARAM_HOSTNAME  /* char* */,
   NDPI_PARAM_ISSUER_DN /* char* */,
@@ -186,11 +189,11 @@ typedef enum {
     please update ndpi_check_flow_risk_exceptions()
     (in ndpi_utils.c) whenever you add a new parameter
   */
-  
+
   /* Leave this as last member */
   NDPI_MAX_RISK_PARAM_ID
 } ndpi_risk_param_id;
-  
+
 typedef struct {
   ndpi_risk_param_id id;
   void *value; /* char* for strings, u_int32_t* for IPv4 addresses */
@@ -272,7 +275,7 @@ typedef struct ndpi_protocol_bitmask_struct {
 struct ndpi_detection_module_struct;
 
 /* NDPI_DEBUG_FUNCTION_PTR (cast) */
-typedef void (*ndpi_debug_function_ptr) (u_int32_t protocol, struct ndpi_detection_module_struct *module_struct,
+typedef void (*ndpi_debug_function_ptr) (u_int16_t protocol, struct ndpi_detection_module_struct *module_struct,
 					 ndpi_log_level_t log_level, const char *file,
 					 const char *func, unsigned line,
 					 const char *format, ...);
@@ -644,7 +647,7 @@ struct ndpi_gre_basehdr {
  * Optional information about flow management (per packet)
  */
 struct ndpi_flow_input_info {
-  unsigned char in_pkt_dir;
+  unsigned char in_pkt_dir; /* If unknown, the library might *returns* to the application the direction calculated internally */
   unsigned char seen_flow_beginning;
 };
 
@@ -748,7 +751,7 @@ typedef enum {
   NDPI_LRUCACHE_MINING,
   NDPI_LRUCACHE_MSTEAMS,
   NDPI_LRUCACHE_FPC_DNS, /* FPC DNS cache */
-
+  NDPI_LRUCACHE_SIGNAL,
   NDPI_LRUCACHE_MAX	/* Last one! */
 } lru_cache_type;
 
@@ -791,9 +794,63 @@ struct ndpi_lru_cache {
 /* Ookla */
 #define NDPI_AGGRESSIVENESS_OOKLA_TLS			0x01 /* Enable detection over TLS (using ookla cache) */
 
+/* OpenVPN */
+#define NDPI_HEURISTICS_OPENVPN_OPCODE			0x01 /* Enable heuristic based on opcode frequency */
+
+/* TLS */
+#define NDPI_HEURISTICS_TLS_OBFUSCATED_PLAIN		0x01 /* Enable heuristic to detect proxied/obfuscated TLS flows over generic/unknown flows */
+#define NDPI_HEURISTICS_TLS_OBFUSCATED_TLS		0x02 /* Enable heuristic to detect proxied/obfuscated TLS flows over TLS tunnels, i.e. TLS over TLS */
+#define NDPI_HEURISTICS_TLS_OBFUSCATED_HTTP		0x04 /* Enable heuristic to detect proxied/obfuscated TLS flows over HTTP/WebSocket */
+
+
 /* ************************************************** */
 
 struct ndpi_flow_tcp_struct {
+  /* TCP sequence number */
+  u_int32_t next_tcp_seq_nr[2];
+  u_int16_t last_tcp_pkt_payload_len;
+
+  /* Part of the TCP header */
+  u_int8_t cli2srv_tcp_flags, srv2cli_tcp_flags;
+  u_int32_t seen_syn:1, seen_syn_ack:1, seen_ack:1;
+
+  /* NDPI_PROTOCOL_IRC */
+  u_int32_t irc_3a_counter:3;
+
+  /* NDPI_PROTOCOL_USENET */
+  u_int32_t usenet_stage:2;
+
+  /* NDPI_PROTOCOL_HTTP */
+  u_int32_t http_stage:3;
+  u_int32_t http_asymmetric_stage:2;
+
+  /* NDPI_PROTOCOL_GNUTELLA */
+  u_int32_t gnutella_stage:2;                  // 0 - 2
+
+  /* NDPI_PROTOCOL_SSH */
+  u_int32_t ssh_stage:3;
+
+  /* NDPI_PROTOCOL_VNC */
+  u_int32_t vnc_stage:2;                        // 0 - 3
+
+  /* NDPI_PROTOCOL_TELNET */
+  u_int32_t telnet_stage:2;                     // 0 - 2
+
+  /* NDPI_PROTOCOL_RADMIN */
+  u_int32_t radmin_stage:1;
+
+  /* NDPI_PROTOCOL_FTP_CONTROL */
+  u_int32_t ftp_control_stage:2;
+
+  /* NDPI_PROTOCOL_SOAP */
+  u_int32_t soap_stage:1;
+
+  /* NDPI_PROTOCOL_SOCKS */
+  u_int32_t socks5_stage:2, socks4_stage:2;
+
+  /* NDPI_PROTOCOL_Z3950 */
+  u_int32_t z3950_stage:2;
+
   /* NDPI_PROTOCOL_MAIL_SMTP */
   /* NDPI_PROTOCOL_MAIL_POP */
   /* NDPI_PROTOCOL_MAIL_IMAP */
@@ -804,6 +861,9 @@ struct ndpi_flow_tcp_struct {
     char username[32], password[16];
   } ftp_imap_pop_smtp;
 
+  /* NDPI_PROTOCOL_LOTUS_NOTES */
+  u_int8_t lotus_notes_packet_id;
+
   /* NDPI_PROTOCOL_MAIL_SMTP */
   u_int16_t smtp_command_bitmask;
 
@@ -819,27 +879,11 @@ struct ndpi_flow_tcp_struct {
   /* NDPI_PROTOCOL_GNUTELLA */
   u_int8_t gnutella_msg_id[3];
 
-  /* NDPI_PROTOCOL_IRC */
-  u_int32_t irc_3a_counter:3;
+  /* NDPI_PROTOCOL_NEST_LOG_SINK */
+  u_int8_t nest_log_sink_matches;
 
-  /* NDPI_PROTOCOL_USENET */
-  u_int32_t usenet_stage:2;
-
-  /* NDPI_PROTOCOL_HTTP */
-  u_int32_t http_stage:3;
-  u_int32_t http_asymmetric_stage:2;
-
-  /* NDPI_PROTOCOL_GNUTELLA */
-  u_int32_t gnutella_stage:2;		       // 0 - 2
-
-  /* NDPI_PROTOCOL_SSH */
-  u_int32_t ssh_stage:3;
-
-  /* NDPI_PROTOCOL_VNC */
-  u_int32_t vnc_stage:2;			// 0 - 3
-
-  /* NDPI_PROTOCOL_TELNET */
-  u_int32_t telnet_stage:2;			// 0 - 2
+  /* NDPI_PROTOCOL_MEMCACHED */
+  u_int8_t memcached_matches;
 
   struct {
     /* NDPI_PROTOCOL_TLS */
@@ -848,46 +892,28 @@ struct ndpi_flow_tcp_struct {
     int16_t tls_application_blocks_len[NDPI_MAX_NUM_TLS_APPL_BLOCKS]; /* + = src->dst, - = dst->src */
   } tls;
 
+  /* NDPI_PROTOCOL_ZMQ */
+  u_char prev_zmq_pkt[10];
+  u_int8_t prev_zmq_pkt_len;
+
+  /* NDPI_PROTOCOL_RTMP */
+  u_int16_t rtmp_client_buffer_len;
+  u_int32_t rtmp_stage:2;
+
   /* NDPI_PROTOCOL_POSTGRES */
   u_int32_t postgres_stage:3;
 
-  /* Part of the TCP header. */
-  u_int32_t seen_syn:1, seen_syn_ack:1, seen_ack:1, __notused:29;
-  u_int8_t cli2srv_tcp_flags, srv2cli_tcp_flags;
-  
   /* NDPI_PROTOCOL_ICECAST */
   u_int32_t icecast_stage:1;
 
   /* NDPI_PROTOCOL_DOFUS */
   u_int32_t dofus_stage:1;
 
-  /* NDPI_PROTOCOL_WORLDOFWARCRAFT */
-  u_int32_t wow_stage:2;
-
   /* NDPI_PROTOCOL_MAIL_POP */
   u_int32_t mail_pop_stage:2;
 
   /* NDPI_PROTOCOL_MAIL_IMAP */
   u_int32_t mail_imap_stage:3, mail_imap_starttls:2;
-
-  /* NDPI_PROTOCOL_SOAP */
-  u_int32_t soap_stage:1;
-
-  /* NDPI_PROTOCOL_LOTUS_NOTES */
-  u_int8_t lotus_notes_packet_id;
-
-  /* NDPI_PROTOCOL_ZMQ */
-  u_int8_t prev_zmq_pkt_len;
-  u_char prev_zmq_pkt[10];
-
-  /* NDPI_PROTOCOL_MEMCACHED */
-  u_int8_t memcached_matches;
-
-  /* NDPI_PROTOCOL_NEST_LOG_SINK */
-  u_int8_t nest_log_sink_matches;
-
-  /* NDPI_PROTOCOL_RADMIN */
-  u_int32_t radmin_stage:1;
 };
 
 /* ************************************************** */
@@ -903,6 +929,7 @@ struct ndpi_flow_udp_struct {
   u_int32_t xbox_stage:1;
 
   /* NDPI_PROTOCOL_QUIC */
+  u_int32_t quic_server_cid_stage:2;
   u_int32_t quic_0rtt_found:1;
   u_int32_t quic_vn_pair:1;
 
@@ -912,38 +939,47 @@ struct ndpi_flow_udp_struct {
   /* NDPI_PROTOCOL_ZOOM */
   u_int32_t zoom_p2p:1;
 
-  /* NDPI_PROTOCOL_EPICGAMES */
-  u_int32_t epicgames_stage:1;
-  u_int32_t epicgames_word;
-
   /* NDPI_PROTOCOL_RAKNET */
   u_int32_t raknet_custom:1;
 
-  /* NDPI_PROTOCOL_EAQ */
-  u_int8_t eaq_pkt_id;
-  u_int32_t eaq_sequence;
+  /* NDPI_PROTOCOL_MUMBLE */
+  u_int32_t mumble_stage:1;
+
+  /* NDPI_PROTOCOL_EPICGAMES */
+  u_int32_t epicgames_stage:1;
+  u_int32_t epicgames_word;
 
   /* NDPI_PROTOCOL_RX */
   u_int32_t rx_conn_epoch;
   u_int32_t rx_conn_id;
 
+  /* NDPI_PROTOCOL_WIREGUARD */
+  u_int32_t wireguard_peer_index[2];
+  u_int8_t wireguard_stage;
+
   /* NDPI_PROTOCOL_MEMCACHED */
   u_int8_t memcached_matches;
 
-  /* NDPI_PROTOCOL_WIREGUARD */
-  u_int8_t wireguard_stage;
-  u_int32_t wireguard_peer_index[2];
+  /* NDPI_PROTOCOL_EAQ */
+  u_int8_t eaq_pkt_id;
+  u_int32_t eaq_sequence;
+
+  /* NDPI_PROTOCOL_MUMBLE */
+  u_int64_t mumble_ident;
 
   /* NDPI_PROTOCOL_QUIC */
   u_int8_t *quic_reasm_buf;
   u_int8_t *quic_reasm_buf_bitmap;
   u_int32_t quic_reasm_buf_last_pos;
+#define QUIC_SERVER_CID_HEURISTIC_LENGTH	8
+  u_int8_t quic_server_cid[QUIC_SERVER_CID_HEURISTIC_LENGTH];
+  u_int8_t quic_client_last_byte;
   /* DCID of the first Initial sent by the client */
   u_int8_t quic_orig_dest_conn_id[20]; /* Max length is 20 on all QUIC versions */
   u_int8_t quic_orig_dest_conn_id_len;
 
   /* NDPI_PROTOCOL_RDP */
-  u_int8_t rdp_to_srv[3], rdp_from_srv[3], rdp_to_srv_pkts, rdp_from_srv_pkts;   
+  u_int8_t rdp_to_srv[3], rdp_from_srv[3], rdp_to_srv_pkts, rdp_from_srv_pkts;
 
   /* NDPI_PROTOCOL_IMO */
   u_int8_t imo_last_one_byte_pkt, imo_last_byte;
@@ -956,9 +992,6 @@ struct ndpi_flow_udp_struct {
   u_int16_t tftp_data_num;
   u_int16_t tftp_ack_num;
 
-  /* NDPI_PROTOCOL_MUMBLE */
-  u_int8_t mumble_stage:1;
-  u_int64_t mumble_ident;
 };
 
 /* ************************************************** */
@@ -1027,7 +1060,12 @@ typedef enum {
 
 #define NUM_BREEDS (NDPI_PROTOCOL_UNRATED+1)
 
-/* Abstract categories to group the protocols. */
+/*
+  Abstract categories to group the protocols.
+
+  IMPORTANT
+  Keep in sync with categories[] on ndpi_main.c
+*/
 typedef enum {
   NDPI_PROTOCOL_CATEGORY_UNSPECIFIED = 0,   /* For general services and unknown protocols */
   NDPI_PROTOCOL_CATEGORY_MEDIA,             /* Multimedia and streaming */
@@ -1049,14 +1087,14 @@ typedef enum {
   NDPI_PROTOCOL_CATEGORY_STREAMING,         /* Streaming protocols */
   NDPI_PROTOCOL_CATEGORY_SYSTEM_OS,         /* System/Operating System level applications */
   NDPI_PROTOCOL_CATEGORY_SW_UPDATE,         /* Software update */
-  
+
   /* See #define NUM_CUSTOM_CATEGORIES */
   NDPI_PROTOCOL_CATEGORY_CUSTOM_1,          /* User custom category 1 */
   NDPI_PROTOCOL_CATEGORY_CUSTOM_2,          /* User custom category 2 */
   NDPI_PROTOCOL_CATEGORY_CUSTOM_3,          /* User custom category 3 */
   NDPI_PROTOCOL_CATEGORY_CUSTOM_4,          /* User custom category 4 */
   NDPI_PROTOCOL_CATEGORY_CUSTOM_5,          /* User custom category 5 */
-  
+
   /* Further categories... */
   NDPI_PROTOCOL_CATEGORY_MUSIC,
   NDPI_PROTOCOL_CATEGORY_VIDEO,
@@ -1076,7 +1114,7 @@ typedef enum {
   NDPI_PROTOCOL_CATEGORY_VIRTUAL_ASSISTANT,
   NDPI_PROTOCOL_CATEGORY_CYBERSECURITY,
   NDPI_PROTOCOL_CATEGORY_ADULT_CONTENT,
-  
+
   /* Some custom categories */
   CUSTOM_CATEGORY_MINING           = 99,
   CUSTOM_CATEGORY_MALWARE          = 100,
@@ -1090,15 +1128,29 @@ typedef enum {
     to track malware, spam etc.
   */
   CUSTOM_CATEGORY_ANTIMALWARE      = 105,
-  
-  /*
-    Crypto Currency e.g Bitcoin, Litecoin, Etherum ..et.
-  */
-  NDPI_PROTOCOL_CATEGORY_CRYPTO_CURRENCY = 106,
+
+  /* Crypto Currency e.g Bitcoin, Litecoin, Etherum ..et. */
+  NDPI_PROTOCOL_CATEGORY_CRYPTO_BLOCKCHAIN = 106,
 
   /* Gambling websites */
   NDPI_PROTOCOL_CATEGORY_GAMBLING = 107,
-  
+  NDPI_PROTOCOL_CATEGORY_HEALTH,
+  NDPI_PROTOCOL_CATEGORY_ARTIFICIAL_INTELLIGENCE,
+  NDPI_PROTOCOL_CATEGORY_FINANCE,
+  NDPI_PROTOCOL_CATEGORY_NEWS,
+  NDPI_PROTOCOL_CATEGORY_SPORT,
+  NDPI_PROTOCOL_CATEGORY_BUSINESS,
+  NDPI_PROTOCOL_CATEGORY_INTERNET_HOSTING,          /* Internet sites including hosting */
+  NDPI_PROTOCOL_CATEGORY_BLOCKCHAIN_CRYPTO,
+  NDPI_PROTOCOL_CATEGORY_BLOG_FORUM,
+  NDPI_PROTOCOL_CATEGORY_GOVERNMENT,
+  NDPI_PROTOCOL_CATEGORY_EDUCATION,
+  NDPI_PROTOCOL_CATEGORY_CND_PROXY,
+  NDPI_PROTOCOL_CATEGORY_HARDWARE_SOFTWARE,
+  NDPI_PROTOCOL_CATEGORY_DATING,
+  NDPI_PROTOCOL_CATEGORY_TRAVEL,
+  NDPI_PROTOCOL_CATEGORY_FOOD,
+
   /*
     IMPORTANT
 
@@ -1117,6 +1169,24 @@ typedef enum {
   NDPI_PROTOCOL_ANY_CATEGORY /* Used to handle wildcards */
 } ndpi_protocol_category_t;
 
+typedef enum {
+  NDPI_PROTOCOL_QOE_CATEGORY_UNSPECIFIED = 0,    /* Unspecified/default QoE constraints          */
+  NDPI_PROTOCOL_QOE_CATEGORY_VOIP_CALLS,         /* VoIP Calls (G.711, RTP, SIP)                 */
+  NDPI_PROTOCOL_QOE_CATEGORY_VIDEO_CONFERENCING, /* Video Conferencing (Zoom, WebRTC, Skype)     */
+  NDPI_PROTOCOL_QOE_CATEGORY_ONLINE_GAMING,      /* Online Gaming (FPS, MOBA, MMO)               */
+  NDPI_PROTOCOL_QOE_CATEGORY_CLOUD_GAMING,       /* Cloud Gaming (GeForce NOW, Stadia, xCloud)   */
+  NDPI_PROTOCOL_QOE_CATEGORY_WEB_BROWSING,       /* Web Browsing (HTTP, HTTPS)                   */
+  NDPI_PROTOCOL_QOE_CATEGORY_DNS_QUERIES,        /* DNS Queries (UDP/53)                         */
+  NDPI_PROTOCOL_QOE_CATEGORY_LIVE_STREAMING,     /* Live Streaming (Twitch, YouTube Live, etc.)  */
+  NDPI_PROTOCOL_QOE_CATEGORY_BUFFERED_STREAMING, /* Buffered Streaming (Netflix, YouTube, etc.)  */
+  NDPI_PROTOCOL_QOE_CATEGORY_REMOTE_ACCESS,      /* Remote Desktop (RDP, VNC, AnyDesk)           */
+  NDPI_PROTOCOL_QOE_CATEGORY_HFT,                /* High-Frequency Trading (HFT, Stock Trading)  */
+  NDPI_PROTOCOL_QOE_CATEGORY_IOT_OT,             /* IoT Smart Home (Zigbee, Z-Wave, MQTT)        */
+  NDPI_PROTOCOL_QOE_CATEGORY_AV,                 /* Autonomous Vehicles (V2X, 5G C-V2X)          */
+
+  NDPI_PROTOCOL_QOE_CATEGORY_MAX                 /* Leave it as last entry !!!                   */
+} ndpi_protocol_qoe_category_t;
+
 /* ntop extensions */
 typedef struct ndpi_proto_defaults {
   char *protoName;
@@ -1127,6 +1197,7 @@ typedef struct ndpi_proto_defaults {
   u_int16_t protoId, protoIdx;
   u_int16_t tcp_default_ports[MAX_DEFAULT_PORTS], udp_default_ports[MAX_DEFAULT_PORTS];
   ndpi_protocol_breed_t protoBreed;
+  ndpi_protocol_qoe_category_t qoeCategory;
   void (*func) (struct ndpi_detection_module_struct *, struct ndpi_flow_struct *flow);
 } ndpi_proto_defaults_t;
 
@@ -1138,24 +1209,28 @@ typedef struct _ndpi_automa {
 
 typedef void ndpi_str_hash;
 
-struct ndpi_fpc_info {
-  u_int16_t master_protocol;
-  u_int16_t app_protocol;
-  ndpi_fpc_confidence_t confidence;
-};
-
-typedef struct ndpi_proto {
+typedef struct {
   /*
     Note
     below we do not use ndpi_protocol_id_t as users can define their own
     custom protocols and thus the typedef could be too short in size.
   */
-  u_int16_t master_protocol /* e.g. HTTP */, app_protocol /* e.g. FaceBook */, protocol_by_ip;
+  u_int16_t master_protocol /* e.g. HTTP */, app_protocol /* e.g. FaceBook */;
+} ndpi_master_app_protocol;
+
+struct ndpi_fpc_info {
+  ndpi_master_app_protocol proto;
+  ndpi_fpc_confidence_t confidence;
+};
+
+typedef struct ndpi_proto {
+  ndpi_master_app_protocol proto;
+  u_int16_t protocol_by_ip;
   ndpi_protocol_category_t category;
   void *custom_category_userdata;
 } ndpi_protocol;
 
-#define NDPI_PROTOCOL_NULL { NDPI_PROTOCOL_UNKNOWN , NDPI_PROTOCOL_UNKNOWN , NDPI_PROTOCOL_UNKNOWN, NDPI_PROTOCOL_CATEGORY_UNSPECIFIED, NULL }
+#define NDPI_PROTOCOL_NULL { { NDPI_PROTOCOL_UNKNOWN , NDPI_PROTOCOL_UNKNOWN }, NDPI_PROTOCOL_UNKNOWN, NDPI_PROTOCOL_CATEGORY_UNSPECIFIED, NULL }
 
 #define NUM_CUSTOM_CATEGORIES      5
 #define CUSTOM_CATEGORY_LABEL_LEN 32
@@ -1165,7 +1240,7 @@ typedef void ndpi_bitmap64;
 typedef void ndpi_bitmap64_fuse; /* probabilistic */
 typedef void ndpi_bitmap_iterator;
 typedef void ndpi_filter;
-    
+
 typedef struct {
   u_int32_t num_allocated_entries, num_used_entries;
   struct ndpi_binary_bitmap_entry *entries;
@@ -1185,6 +1260,7 @@ typedef enum {
 } ndpi_cipher_weakness;
 
 #define MAX_NUM_TLS_SIGNATURE_ALGORITHMS 16
+#define MAX_NUM_DNS_RSP_ADDRESSES         4
 
 typedef struct {
   union {
@@ -1195,7 +1271,7 @@ typedef struct {
   u_int16_t port;
   u_int16_t is_ipv6: 1, _pad: 15;
 } ndpi_address_port;
-  
+
 struct tls_heuristics {
   /*
     TLS heuristics for detecting browsers usage
@@ -1206,16 +1282,53 @@ struct tls_heuristics {
 
 struct ndpi_risk_information {
   ndpi_risk_enum id;
-  char *info;  
+  char *info;
+};
+
+struct ndpi_metadata_monitoring {
+  union {
+    struct {
+      ndpi_address_port mapped_address;
+      ndpi_address_port peer_address;
+      ndpi_address_port relayed_address;
+      ndpi_address_port response_origin;
+      ndpi_address_port other_address;
+    } dtls_stun_rtp;
+  } protos;
+};
+
+typedef enum {
+  ndpi_os_unknown     = 0,
+  ndpi_os_windows     = 1,
+  ndpi_os_macos       = 2,
+  ndpi_os_ios_ipad_os = 3,
+  ndpi_os_android     = 4,
+  ndpi_os_linux       = 5,
+  ndpi_os_freebsd     = 6,
+  ndpi_os_MAX_OS      = 7 /* Keep it as last */
+} ndpi_os;
+
+struct os_fingerprint {
+  const char *fingerprint;
+  ndpi_os os;
+};
+
+struct rtp_info {
+  u_int8_t payload_type;
+  bool payload_detected;
+  u_int32_t evs_subtype;
 };
 
 struct ndpi_flow_struct {
   u_int16_t detected_protocol_stack[NDPI_PROTOCOL_SIZE];
 
-  /* init parameter, internal used to set up timestamp,... */
-  u_int16_t guessed_protocol_id, guessed_protocol_id_by_ip, guessed_category, guessed_header_category;
+  u_int16_t guessed_protocol_id;       /* Classification by-port. Set with the first pkt and never updated */
+  u_int16_t guessed_protocol_id_by_ip; /* Classification by-ip. Set with the first pkt and never updated */
+  u_int16_t fast_callback_protocol_id; /* Partial/incomplete classification. Used internally as first callback when iterating all the protocols */
+  u_int16_t guessed_header_category;
   u_int8_t l4_proto, protocol_id_already_guessed:1, fail_with_unknown:1,
     init_finished:1, client_packet_direction:1, packet_direction:1, is_ipv6:1, first_pkt_fully_encrypted:1, skip_entropy_check: 1;
+  u_int8_t monitoring:1, already_gaveup:1, _pad:6;
 
   u_int16_t num_dissector_calls;
   ndpi_confidence_t confidence; /* ndpi_confidence_t */
@@ -1223,12 +1336,6 @@ struct ndpi_flow_struct {
   /* First Packet Classification info */
   struct ndpi_fpc_info fpc;
 
-  /*
-    if ndpi_struct->direction_detect_disable == 1
-    tcp sequence number connection tracking
-  */
-  u_int32_t next_tcp_seq_nr[2];
-
   /* Flow addresses (useful for LRU lookups in ndpi_detection_giveup())
      and ports. All in *network* byte order.
      Client and server.
@@ -1238,7 +1345,7 @@ struct ndpi_flow_struct {
     u_int8_t v6[16];
   } c_address, s_address;	/* For some unknown reasons, x86_64-w64-mingw32-gcc doesn't like the name "s_addr" */
   u_int16_t c_port, s_port;
-  
+
   // -----------------------------------------
 
   u_int8_t max_extra_packets_to_check;
@@ -1249,6 +1356,15 @@ struct ndpi_flow_struct {
 
   u_int64_t last_packet_time_ms;
 
+  ndpi_protocol_category_t category;
+
+  /* Counters with only packets with L5 data (ie no TCP SYN, pure ACKs, ...) */
+  u_int16_t packet_counter;
+  u_int16_t packet_direction_counter[2];
+  /* Counters with all packets even those without payload */
+  u_int16_t all_packets_counter;
+  u_int16_t packet_direction_complete_counter[2];
+
   /*
     the tcp / udp / other l4 value union
     used to reduce the number of bytes for tcp or udp protocol states
@@ -1275,7 +1391,12 @@ struct ndpi_flow_struct {
   ndpi_risk risk, risk_shadow; /* Issues found with this flow [bitmask of ndpi_risk] */
   struct ndpi_risk_information risk_infos[MAX_NUM_RISK_INFOS]; /* String that contains information about the risks found */
   u_int8_t num_risk_infos;
-  
+
+  struct {
+    char *fingerprint;
+    ndpi_os os_hint;
+  } tcp;
+
   /*
     This structure below will not not stay inside the protos
     structure below as HTTP is used by many subprotocols
@@ -1286,14 +1407,16 @@ struct ndpi_flow_struct {
   struct {
     ndpi_http_method method;
     u_int8_t request_version; /* 0=1.0 and 1=1.1. Create an enum for this? */
+    u_int8_t websocket:1, request_header_observed:1, first_payload_after_header_observed:1, is_form:1, _pad:4;
     u_int16_t response_status_code; /* 200, 404, etc. */
-    char *url, *content_type /* response */, *request_content_type /* e.g. for POST */, *user_agent, *server;
+    char *url, *content_type /* response */, *request_content_type /* e.g. for POST */, *user_agent, *server, *referer, *host;
     char *detected_os; /* Via HTTP/QUIC User-Agent */
     char *nat_ip; /* Via HTTP X-Forwarded-For */
     char *filename; /* Via HTTP Content-Disposition */
+    char *username, *password;
   } http;
 
-  ndpi_multimedia_flow_type flow_multimedia_type;
+  u_int8_t flow_multimedia_types;
 
   /*
      Put outside of the union to avoid issues in case the protocol
@@ -1306,29 +1429,38 @@ struct ndpi_flow_struct {
   } kerberos_buf;
 
   struct {
-    u_int8_t maybe_dtls : 1, is_turn : 1, pad : 6;
+    u_int8_t maybe_dtls:1, rtcp_seen:1, is_turn : 1, is_client_controlling:1, pad : 4;
     ndpi_address_port mapped_address, peer_address, relayed_address, response_origin, other_address;
+    u_int8_t num_xor_relayed_addresses, num_xor_mapped_addresses;
+    u_int8_t num_non_stun_pkt, non_stun_pkt_len[2];
+    u_int16_t rtp_counters[2];
+    u_int32_t t_start, t_end;
   } stun;
 
   struct {
     message_t message[2]; /* Directions */
-    u_int8_t certificate_processed:1, _pad:7;
+    u_int8_t certificate_processed:1, change_cipher_from_client:1, change_cipher_from_server:1, from_opportunistic_tls:1, from_rdp:1, pad:3;
+    struct tls_obfuscated_heuristic_state *obfuscated_heur_state;
   } tls_quic; /* Used also by DTLS and POPS/IMAPS/SMTPS/FTPS */
 
+  struct rtp_info rtp[2 /* directions */];
+
   union {
     /* the only fields useful for nDPI and ntopng */
     struct {
-      u_int8_t num_queries, num_answers, reply_code;
-      u_int8_t is_query:1, is_rsp_addr_ipv6:1, pad:6;
-      u_int16_t query_type, query_class, rsp_type, edns0_udp_payload_size;
-      ndpi_ip_addr_t rsp_addr; /* The first address in a DNS response packet (A and AAAA) */
+      u_int8_t num_queries, num_answers, reply_code, num_rsp_addr;
+      u_int8_t is_query:1, pad:7;
+      u_int16_t transaction_id, query_type, query_class, rsp_type, edns0_udp_payload_size;
+      u_int8_t is_rsp_addr_ipv6[MAX_NUM_DNS_RSP_ADDRESSES];
+      ndpi_ip_addr_t rsp_addr[MAX_NUM_DNS_RSP_ADDRESSES]; /* The first num_rsp_addr address in a DNS response packet (A and AAAA) */
+      u_int32_t rsp_addr_ttl[MAX_NUM_DNS_RSP_ADDRESSES];
       char geolocation_iata_code[4];
       char ptr_domain_name[64 /* large enough but smaller than { } tls */];
     } dns;
 
     struct {
-      u_int8_t request_code;
       u_int8_t version;
+      u_int8_t mode;
     } ntp;
 
     struct {
@@ -1344,15 +1476,15 @@ struct ndpi_flow_struct {
 
     struct {
       char currency[16];
-    } mining;  
+    } mining;
 
     struct {
       char *server_names, *advertised_alpns, *negotiated_alpn, *tls_supported_versions, *issuerDN, *subjectDN;
       u_int32_t notBefore, notAfter;
-      char ja3_client[33], ja3_server[33], ja4_client[37];
+      char ja3_server[33], ja4_client[37], *ja4_client_raw;
       u_int16_t server_cipher;
       u_int8_t sha1_certificate_fingerprint[20];
-      u_int8_t hello_processed:1, ch_direction:1, subprotocol_detected:1, fingerprint_set:1, webrtc:1, _pad:3;
+      u_int8_t client_hello_processed:1, ch_direction:1, subprotocol_detected:1, server_hello_processed:1, fingerprint_set:1, webrtc:1, _pad:2;
 
 #ifdef TLS_HANDLE_SIGNATURE_ALGORITMS
       /* Under #ifdef to save memory for those who do not need them */
@@ -1364,11 +1496,6 @@ struct ndpi_flow_struct {
 
       u_int16_t ssl_version, server_names_len;
 
-      struct {
-        u_int16_t cipher_suite;
-        char *esni;
-      } encrypted_sni;
-
       struct {
         u_int16_t version;
       } encrypted_ch;
@@ -1376,6 +1503,7 @@ struct ndpi_flow_struct {
       ndpi_cipher_weakness server_unsafe_cipher;
 
       u_int32_t quic_version;
+      u_int32_t quic_idle_timeout_sec;
     } tls_quic; /* Used also by DTLS and POPS/IMAPS/SMTPS/FTPS */
 
     struct {
@@ -1410,7 +1538,7 @@ struct ndpi_flow_struct {
     } discord;
 
     struct {
-      char version[32];
+      char version[64];
     } ubntac2;
 
     /* In TLS.Bittorent flows there is no hash.
@@ -1422,6 +1550,7 @@ struct ndpi_flow_struct {
     } bittorrent;
 
     struct {
+      char options[48];
       char fingerprint[48];
       char class_ident[48];
     } dhcp;
@@ -1455,24 +1584,57 @@ struct ndpi_flow_struct {
       u_int8_t url_count;
       char url[4][48];
     } slp;
+
+    struct {
+      char *from;
+      char from_imsi[16]; /* IMSI is 15 digit long, at most; + 1 for NULL terminator */
+      char *to;
+      char to_imsi[16];
+    } sip;
+
+    struct {
+      char mac_addr[6], identity[16], version[48], sw_id[16], board[32], iface_name[32];
+      u_int32_t ipv4_addr, uptime;
+      struct ndpi_in6_addr ipv6_addr;
+    } mikrotik;
+
+    struct {
+      char *method;
+      char *bootid;
+      char *usn;
+      char *cache_controle;
+      char *location;
+      char *household_smart_speaker_audio;
+      char *rincon_household;
+      char *rincon_bootseq;
+      char *rincon_wifimode;
+      char *rincon_variant;
+      char *sonos_securelocation;
+      char *securelocation_upnp;
+      char *location_smart_speaker_audio;
+      char *nt;
+      char *nts;
+      char *server;
+      char *man;
+      char *mx;
+      char *st;
+      char *user_agent;
+    } ssdp;
+
+    struct {
+      ndpi_http_method method;
+      char user_agent[32];
+      char url[64];
+    } fast_cgi;
+
   } protos;
 
-  /*** ALL protocol specific 64 bit variables here ***/
+  /* **Packet** metadata for flows where monitoring is enabled. It is reset after each packet! */
+  struct ndpi_metadata_monitoring *monit;
 
   /* protocols which have marked a connection as this connection cannot be protocol XXX, multiple u_int64_t */
   NDPI_PROTOCOL_BITMASK excluded_protocol_bitmask;
 
-  ndpi_protocol_category_t category;
-
-  /* Only packets with L5 data (ie no TCP SYN, pure ACKs, ...) */
-  u_int16_t packet_counter;		      // can be 0 - 65000
-  u_int16_t packet_direction_counter[2];
-  u_int8_t  packet_direction_with_payload_observed[2]; /* 0 = no packet with payload observed, 1 = at least one packet with payload observed */
-
-  /* All packets even those without payload */
-  u_int16_t all_packets_counter;
-  u_int16_t packet_direction_complete_counter[2];      // can be 0 - 65000
-
   /* NDPI_PROTOCOL_BITTORRENT */
   u_int8_t bittorrent_stage;		      // can be 0 - 255
   u_int8_t bt_check_performed : 1;
@@ -1483,21 +1645,6 @@ struct ndpi_flow_struct {
   /* NDPI_PROTOCOL_ZATTOO */
   u_int8_t zattoo_stage:3;
 
-  /* NDPI_PROTOCOL_SOCKS */
-  u_int8_t socks5_stage:2, socks4_stage:2;      // 0 - 3
-
-  /* NDPI_PROTOCOL_FTP_CONTROL */
-  u_int8_t ftp_control_stage:2;
-
-  /* NDPI_PROTOCOL_RTMP */
-  u_int8_t rtmp_stage:2;
-
-  /* NDPI_PROTOCOL_STARCRAFT */
-  u_int8_t starcraft_udp_stage : 3;	// 0-7
-
-  /* NDPI_PROTOCOL_Z3950 */
-  u_int8_t z3950_stage : 2; // 0-3
-
   /* NDPI_PROTOCOL_OOKLA */
   u_int8_t ookla_stage : 1;
 
@@ -1510,11 +1657,19 @@ struct ndpi_flow_struct {
 
   /* NDPI_PROTOCOL_OPENVPN */
   u_int8_t ovpn_session_id[2][8];
+  u_int8_t ovpn_alg_standard_state : 2;
+  u_int8_t ovpn_alg_heur_opcode_state : 2;
+  u_int8_t ovpn_heur_opcode__codes_num : 4;
+  u_int8_t ovpn_heur_opcode__num_msgs;
+#define OPENVPN_HEUR_MAX_NUM_OPCODES 4
+  u_int8_t ovpn_heur_opcode__codes[OPENVPN_HEUR_MAX_NUM_OPCODES];
+  u_int8_t ovpn_heur_opcode__resets[2];
+  u_int16_t ovpn_heur_opcode__missing_bytes[2];
 
   /* NDPI_PROTOCOL_TINC */
   u_int8_t tinc_state;
- 
-   /* NDPI_PROTOCOL_RTCP */ 
+
+   /* NDPI_PROTOCOL_RTCP */
    u_int8_t rtcp_stage:2;
 
    /* NDPI_PROTOCOL_RTP */
@@ -1525,8 +1680,8 @@ struct ndpi_flow_struct {
   /* Flow payload */
   u_int16_t flow_payload_len;
   char *flow_payload;
-  
-  /* 
+
+  /*
      Leave this field below at the end
      The field below can be used by third
      party dissectors for storing private data
@@ -1536,11 +1691,11 @@ struct ndpi_flow_struct {
 
 #if !defined(NDPI_CFFI_PREPROCESSING) && defined(__linux__)
 #if defined(__STDC_VERSION__) && __STDC_VERSION__ >= 201112L
-_Static_assert(sizeof(((struct ndpi_flow_struct *)0)->protos) <= 256,
-               "Size of the struct member protocols increased to more than 256 bytes, "
+_Static_assert(sizeof(((struct ndpi_flow_struct *)0)->protos) <= 264,
+               "Size of the struct member protocols increased to more than 264 bytes, "
                "please check if this change is necessary.");
-_Static_assert(sizeof(struct ndpi_flow_struct) <= 1120,
-               "Size of the flow struct increased to more than 1120 bytes, "
+_Static_assert(sizeof(struct ndpi_flow_struct) <= 1232,
+               "Size of the flow struct increased to more than 1232 bytes, "
                "please check if this change is necessary.");
 #endif
 #endif
@@ -1578,7 +1733,8 @@ typedef enum {
   ndpi_serialization_format_tlv,
   ndpi_serialization_format_json,
   ndpi_serialization_format_csv,
-  ndpi_serialization_format_multiline_json
+  ndpi_serialization_format_multiline_json, /* new-line separated records */
+  ndpi_serialization_format_inner_json /* no outer braces */
 } ndpi_serialization_format;
 
 /* Note:
@@ -1608,7 +1764,7 @@ typedef enum {
 } ndpi_serialization_type;
 
 #define NDPI_SERIALIZER_DEFAULT_HEADER_SIZE 1024
-#define NDPI_SERIALIZER_DEFAULT_BUFFER_SIZE 8192
+#define NDPI_SERIALIZER_DEFAULT_BUFFER_SIZE  256
 #define NDPI_SERIALIZER_DEFAULT_BUFFER_INCR 1024
 
 #define NDPI_SERIALIZER_STATUS_COMMA     (1 << 0)
@@ -1619,6 +1775,7 @@ typedef enum {
 #define NDPI_SERIALIZER_STATUS_LIST      (1 << 5)
 #define NDPI_SERIALIZER_STATUS_SOL       (1 << 6)
 #define NDPI_SERIALIZER_STATUS_HDR_DONE  (1 << 7)
+#define NDPI_SERIALIZER_STATUS_CEOB      (1 << 8)
 
 typedef struct {
   u_int32_t size_used;
@@ -1644,6 +1801,7 @@ typedef struct {
   char csv_separator[2];
   u_int8_t has_snapshot;
   u_int8_t multiline_json_array;
+  u_int8_t inner_json;
   ndpi_private_serializer_status snapshot;
 } ndpi_private_serializer;
 
@@ -1666,7 +1824,7 @@ typedef struct {
 
 struct ndpi_analyze_struct {
   u_int64_t *values;
-  u_int64_t min_val, max_val, sum_total;
+  u_int64_t min_val, max_val, sum_total, jitter_total;
   u_int32_t num_data_entries, next_value_insert_index;
   u_int16_t num_values_array_len /* length of the values array */;
 
@@ -1759,6 +1917,22 @@ struct ndpi_bin {
 
 /* **************************************** */
 
+/* Implemented in third_party/src/kdtree.c */
+typedef void ndpi_kd_tree;
+typedef void ndpi_kd_tree_result;
+
+/* Implemented in third_party/src/ball.c */
+typedef void ndpi_btree;
+
+typedef struct {
+  double **distances;
+  int    **indices;
+  int    n_samples;
+  int    n_neighbors;
+} ndpi_knn;
+
+/* **************************************** */
+
 #define HW_HISTORY_LEN               4
 #define MAX_SQUARE_ERROR_ITERATIONS 64 /* MUST be < num_values_rollup (256 max) */
 
@@ -1812,6 +1986,21 @@ struct ndpi_des_struct {
 
 /* **************************************** */
 
+struct ndpi_address_cache_item {
+  ndpi_ip_addr_t addr; /* key */
+  char *hostname; /* value */
+  u_int32_t expire_epoch;
+  struct ndpi_address_cache_item *next; /* Linked list */
+};
+
+struct ndpi_address_cache {
+  u_int32_t num_cached_addresses, num_root_nodes;
+  u_int32_t num_entries, max_num_entries;
+  struct ndpi_address_cache_item **address_cache_root;
+};
+
+/* **************************************** */
+
 /* Prototype used to define custom DGA detection function */
 typedef int (*ndpi_custom_dga_predict_fctn)(const char* domain, int domain_length);
 
